import { Component, Directive, Input, ElementRef, ViewContainerRef, TemplateRef, ContentChild, IterableDiffers, ChangeDetectorRef, EventEmitter, ViewChild, Output, Host, ChangeDetectionStrategy } from "@angular/core";
import { isBlank } from "../lang-facade";
import { isListLikeIterable } from "../collection-facade";
import { ObservableArray } from "data/observable-array";
import { LayoutBase } from "ui/layouts/layout-base";
import { listViewLog } from "../trace";
const NG_VIEW = "_ngViewRef";
export class ListItemContext {
    constructor($implicit, item, index, even, odd) {
        this.$implicit = $implicit;
        this.item = item;
        this.index = index;
        this.even = even;
        this.odd = odd;
    }
}
export let ListViewComponent = class ListViewComponent {
    constructor(_elementRef, _iterableDiffers, _cdr) {
        this._iterableDiffers = _iterableDiffers;
        this._cdr = _cdr;
        this.setupItemView = new EventEmitter();
        this.listView = _elementRef.nativeElement;
        this.listView.on("itemLoading", this.onItemLoading, this);
    }
    get nativeElement() {
        return this.listView;
    }
    get items() {
        return this._items;
    }
    set items(value) {
        this._items = value;
        let needDiffer = true;
        if (value instanceof ObservableArray) {
            needDiffer = false;
        }
        if (needDiffer && !this._differ && isListLikeIterable(value)) {
            this._differ = this._iterableDiffers.find(this._items)
                .create(this._cdr, (_index, item) => { return item; });
        }
        this.listView.items = this._items;
    }
    ngAfterContentInit() {
        listViewLog("ListView.ngAfterContentInit()");
        this.setItemTemplates();
    }
    ngOnDestroy() {
        this.listView.off("itemLoading", this.onItemLoading, this);
    }
    setItemTemplates() {
        if (this._templateMap) {
            listViewLog("Setting templates");
            const templates = [];
            this._templateMap.forEach(value => {
                templates.push(value);
            });
            this.listView.itemTemplates = templates;
        }
    }
    registerTemplate(key, template) {
        listViewLog("registerTemplate for key: " + key);
        if (!this._templateMap) {
            this._templateMap = new Map();
        }
        const keyedTemplate = {
            key,
            createView: () => {
                listViewLog("registerTemplate for key: " + key);
                const viewRef = this.loader.createEmbeddedView(template, new ListItemContext(), 0);
                const resultView = getSingleViewFromViewRef(viewRef);
                resultView[NG_VIEW] = viewRef;
                return resultView;
            }
        };
        this._templateMap.set(key, keyedTemplate);
    }
    onItemLoading(args) {
        if (!args.view && !this.itemTemplate) {
            return;
        }
        let index = args.index;
        let items = args.object.items;
        let currentItem = typeof (items.getItem) === "function" ?
            items.getItem(index) : items[index];
        let viewRef;
        if (args.view) {
            listViewLog("onItemLoading: " + index + " - Reusing existing view");
            viewRef = args.view[NG_VIEW];
            // getting angular view from original element (in cases when ProxyViewContainer
            // is used NativeScript internally wraps it in a StackLayout)
            if (!viewRef) {
                viewRef = (args.view._subViews && args.view._subViews.length > 0) ?
                    args.view._subViews[0][NG_VIEW] : undefined;
            }
        }
        else {
            listViewLog("onItemLoading: " + index + " - Creating view from template");
            viewRef = this.loader.createEmbeddedView(this.itemTemplate, new ListItemContext(), 0);
            args.view = getSingleViewFromViewRef(viewRef);
            args.view[NG_VIEW] = viewRef;
        }
        this.setupViewRef(viewRef, currentItem, index);
        this.detectChangesOnChild(viewRef, index);
    }
    setupViewRef(viewRef, data, index) {
        if (isBlank(viewRef)) {
            return;
        }
        const context = viewRef.context;
        context.$implicit = data;
        context.item = data;
        context.index = index;
        context.even = (index % 2 === 0);
        context.odd = !context.even;
        this.setupItemView.next({ view: viewRef, data: data, index: index, context: context });
    }
    detectChangesOnChild(viewRef, index) {
        // Manually detect changes in child view ref
        // TODO: Is there a better way of getting viewRef"s change detector
        const childChangeDetector = viewRef;
        listViewLog("Manually detect changes in child: " + index);
        childChangeDetector.markForCheck();
        childChangeDetector.detectChanges();
    }
    ngDoCheck() {
        if (this._differ) {
            listViewLog("ngDoCheck() - execute differ");
            const changes = this._differ.diff(this._items);
            if (changes) {
                listViewLog("ngDoCheck() - refresh");
                this.listView.refresh();
            }
        }
    }
};
__decorate([
    ViewChild("loader", { read: ViewContainerRef }), 
    __metadata('design:type', ViewContainerRef)
], ListViewComponent.prototype, "loader", void 0);
__decorate([
    Output(), 
    __metadata('design:type', Object)
], ListViewComponent.prototype, "setupItemView", void 0);
__decorate([
    ContentChild(TemplateRef), 
    __metadata('design:type', TemplateRef)
], ListViewComponent.prototype, "itemTemplate", void 0);
__decorate([
    Input(), 
    __metadata('design:type', Object)
], ListViewComponent.prototype, "items", null);
ListViewComponent = __decorate([
    Component({
        selector: "ListView",
        template: `
        <DetachedContainer>
            <Placeholder #loader></Placeholder>
        </DetachedContainer>`,
        changeDetection: ChangeDetectionStrategy.OnPush
    }), 
    __metadata('design:paramtypes', [ElementRef, IterableDiffers, ChangeDetectorRef])
], ListViewComponent);
function getSingleViewRecursive(nodes, nestLevel) {
    const actualNodes = nodes.filter((n) => !!n && n.nodeName !== "#text");
    if (actualNodes.length === 0) {
        throw new Error("No suitable views found in list template! Nesting level: " + nestLevel);
    }
    else if (actualNodes.length > 1) {
        throw new Error("More than one view found in list template! Nesting level: " + nestLevel);
    }
    else {
        if (actualNodes[0]) {
            let parentLayout = actualNodes[0].parent;
            if (parentLayout instanceof LayoutBase) {
                parentLayout.removeChild(actualNodes[0]);
            }
            return actualNodes[0];
        }
        else {
            return getSingleViewRecursive(actualNodes[0].children, nestLevel + 1);
        }
    }
}
function getSingleViewFromViewRef(viewRef) {
    return getSingleViewRecursive(viewRef.rootNodes, 0);
}
export let TemplateKeyDirective = class TemplateKeyDirective {
    constructor(templateRef, list) {
        this.templateRef = templateRef;
        this.list = list;
    }
    set nsTemplateKey(value) {
        if (this.list && this.templateRef) {
            this.list.registerTemplate(value, this.templateRef);
        }
    }
};
__decorate([
    Input(), 
    __metadata('design:type', Object), 
    __metadata('design:paramtypes', [Object])
], TemplateKeyDirective.prototype, "nsTemplateKey", null);
TemplateKeyDirective = __decorate([
    Directive({ selector: "[nsTemplateKey]" }),
    __param(1, Host()), 
    __metadata('design:paramtypes', [TemplateRef, ListViewComponent])
], TemplateKeyDirective);
//# sourceMappingURL=list-view-comp.js.map