import { Inject, Injectable, Optional, NgZone, Renderer, RenderComponentType } from "@angular/core";
import { APP_ROOT_VIEW, DEVICE } from "./platform-providers";
import { isBlank } from "./lang-facade";
import { View } from "ui/core/view";
import { addCss } from "application";
import { topmost } from "ui/frame";
import { ViewUtil } from "./view-util";
import { rendererLog as traceLog } from "./trace";
import { escapeRegexSymbols } from "utils/utils";
import { NativeScriptAnimationDriver } from "./animation-driver";
// CONTENT_ATTR not exported from dom_renderer - we need it for styles application.
export const COMPONENT_VARIABLE = "%COMP%";
export const CONTENT_ATTR = `_ngcontent-${COMPONENT_VARIABLE}`;
export let NativeScriptRootRenderer = class NativeScriptRootRenderer {
    constructor(_rootView, device, _zone) {
        this._rootView = _rootView;
        this._zone = _zone;
        this._registeredComponents = new Map();
        this._viewUtil = new ViewUtil(device);
    }
    get animationDriver() {
        if (!this._animationDriver) {
            this._animationDriver = new NativeScriptAnimationDriver();
        }
        return this._animationDriver;
    }
    get rootView() {
        if (!this._rootView) {
            this._rootView = topmost().currentPage;
        }
        return this._rootView;
    }
    get page() {
        return this.rootView.page;
    }
    get viewUtil() {
        return this._viewUtil;
    }
    renderComponent(componentProto) {
        let renderer = this._registeredComponents.get(componentProto.id);
        if (isBlank(renderer)) {
            renderer = new NativeScriptRenderer(this, componentProto, this.animationDriver, this._zone);
            this._registeredComponents.set(componentProto.id, renderer);
        }
        return renderer;
    }
};
NativeScriptRootRenderer = __decorate([
    Injectable(),
    __param(0, Optional()),
    __param(0, Inject(APP_ROOT_VIEW)),
    __param(1, Inject(DEVICE)), 
    __metadata('design:paramtypes', [View, Object, NgZone])
], NativeScriptRootRenderer);
export let NativeScriptRenderer = class NativeScriptRenderer extends Renderer {
    constructor(rootRenderer, componentProto, animationDriver, zone) {
        super();
        this.rootRenderer = rootRenderer;
        this.componentProto = componentProto;
        this.animationDriver = animationDriver;
        this.zone = zone;
        this.attrReplacer = new RegExp(escapeRegexSymbols(CONTENT_ATTR), "g");
        this.attrSanitizer = /-/g;
        let stylesLength = this.componentProto.styles.length;
        this.componentProtoId = this.componentProto.id;
        for (let i = 0; i < stylesLength; i++) {
            this.hasComponentStyles = true;
            let cssString = this.componentProto.styles[i] + "";
            const realCSS = this.replaceNgAttribute(cssString, this.componentProtoId);
            addCss(realCSS);
        }
        traceLog("NativeScriptRenderer created");
    }
    get viewUtil() {
        return this.rootRenderer.viewUtil;
    }
    replaceNgAttribute(input, componentId) {
        return input.replace(this.attrReplacer, "_ng_content_" + componentId.replace(this.attrSanitizer, "_"));
    }
    renderComponent(componentProto) {
        return this.rootRenderer.renderComponent(componentProto);
    }
    selectRootElement(selector) {
        traceLog("selectRootElement: " + selector);
        const rootView = this.rootRenderer.rootView;
        rootView.nodeName = "ROOT";
        return rootView;
    }
    createViewRoot(hostElement) {
        traceLog("CREATE VIEW ROOT: " + hostElement.nodeName);
        return hostElement;
    }
    projectNodes(parentElement, nodes) {
        traceLog("NativeScriptRenderer.projectNodes");
        nodes.forEach((node) => {
            this.viewUtil.insertChild(parentElement, node);
        });
    }
    attachViewAfter(anchorNode, viewRootNodes) {
        traceLog("NativeScriptRenderer.attachViewAfter: " + anchorNode.nodeName + " " + anchorNode);
        const parent = (anchorNode.parent || anchorNode.templateParent);
        const insertPosition = this.viewUtil.getChildIndex(parent, anchorNode);
        viewRootNodes.forEach((node, index) => {
            const childIndex = insertPosition + index + 1;
            this.viewUtil.insertChild(parent, node, childIndex);
        });
    }
    detachView(viewRootNodes) {
        traceLog("NativeScriptRenderer.detachView");
        for (let i = 0; i < viewRootNodes.length; i++) {
            let node = viewRootNodes[i];
            this.viewUtil.removeChild(node.parent, node);
        }
    }
    destroyView(_hostElement, _viewAllNodes) {
        traceLog("NativeScriptRenderer.destroyView");
        // Seems to be called on component dispose only (router outlet)
        // TODO: handle this when we resolve routing and navigation.
    }
    setElementProperty(renderElement, propertyName, propertyValue) {
        traceLog("NativeScriptRenderer.setElementProperty " + renderElement + ": " +
            propertyName + " = " + propertyValue);
        this.viewUtil.setProperty(renderElement, propertyName, propertyValue);
    }
    setElementAttribute(renderElement, attributeName, attributeValue) {
        traceLog("NativeScriptRenderer.setElementAttribute " + renderElement + ": " +
            attributeName + " = " + attributeValue);
        return this.setElementProperty(renderElement, attributeName, attributeValue);
    }
    setElementClass(renderElement, className, isAdd) {
        traceLog("NativeScriptRenderer.setElementClass " + className + " - " + isAdd);
        if (isAdd) {
            this.viewUtil.addClass(renderElement, className);
        }
        else {
            this.viewUtil.removeClass(renderElement, className);
        }
    }
    setElementStyle(renderElement, styleName, styleValue) {
        this.viewUtil.setStyleProperty(renderElement, styleName, styleValue);
    }
    // Used only in debug mode to serialize property changes to comment nodes,
    // such as <template> placeholders.
    setBindingDebugInfo(renderElement, propertyName, propertyValue) {
        traceLog("NativeScriptRenderer.setBindingDebugInfo: " + renderElement + ", " +
            propertyName + " = " + propertyValue);
    }
    setElementDebugInfo(renderElement, _info /*RenderDebugInfo*/) {
        traceLog("NativeScriptRenderer.setElementDebugInfo: " + renderElement);
    }
    invokeElementMethod(_renderElement, methodName, args) {
        traceLog("NativeScriptRenderer.invokeElementMethod " + methodName + " " + args);
    }
    setText(_renderNode, _text) {
        traceLog("NativeScriptRenderer.setText");
    }
    createTemplateAnchor(parentElement) {
        traceLog("NativeScriptRenderer.createTemplateAnchor");
        return this.viewUtil.createTemplateAnchor(parentElement);
    }
    createElement(parentElement, name) {
        traceLog("NativeScriptRenderer.createElement: " + name + " parent: " +
            parentElement + ", " + (parentElement ? parentElement.nodeName : "null"));
        return this.viewUtil.createView(name, parentElement, (view) => {
            // Set an attribute to the view to scope component-specific css.
            // The property name is pre-generated by Angular.
            if (this.hasComponentStyles) {
                const cssAttribute = this.replaceNgAttribute(CONTENT_ATTR, this.componentProtoId);
                view[cssAttribute] = true;
            }
        });
    }
    createText(_parentElement, _value) {
        traceLog("NativeScriptRenderer.createText");
        return this.viewUtil.createText();
    }
    listen(renderElement, eventName, callback) {
        traceLog("NativeScriptRenderer.listen: " + eventName);
        // Explicitly wrap in zone
        let zonedCallback = (...args) => {
            this.zone.run(() => {
                callback.apply(undefined, args);
            });
        };
        renderElement.on(eventName, zonedCallback);
        if (eventName === View.loadedEvent && renderElement.isLoaded) {
            const notifyData = { eventName: View.loadedEvent, object: renderElement };
            zonedCallback(notifyData);
        }
        return () => renderElement.off(eventName, zonedCallback);
    }
    listenGlobal(_target, _eventName, _callback) {
        throw new Error("NativeScriptRenderer.listenGlobal() - Not implemented.");
    }
    animate(element, startingStyles, keyframes, duration, delay, easing) {
        let player = this.animationDriver.animate(element, startingStyles, keyframes, duration, delay, easing);
        return player;
    }
};
NativeScriptRenderer = __decorate([
    Injectable(), 
    __metadata('design:paramtypes', [NativeScriptRootRenderer, RenderComponentType, NativeScriptAnimationDriver, NgZone])
], NativeScriptRenderer);
//# sourceMappingURL=renderer.js.map